﻿#include "p16f18877.inc"
__CONFIG _CONFIG1, _FEXTOSC_HS & _RSTOSC_EXT1X
__CONFIG _CONFIG2, _BOREN_ON & _PPS1WAY_OFF
__CONFIG _CONFIG3, _WDTE_OFF 
__CONFIG _CONFIG5, _CP_OFF & _CPD_OFF
    
;---------PIC使用-------------	
IDX		EQU	0X20	 ;查表指標 	
D1 		EQU	0X21	 	 	
D2		EQU	0X22	 	
;---------LCD顯示-------------
LCD_DATA	EQU	0X22
LCD_RS              EQU     0        ;LCD命令/資料控制接腳
LCD_EN              EQU     1        ;LCD致能接腳
LCD_BUS         EQU     LATD    ;LCD資料BUS
LCD_CMD         EQU     LATC    ;LCD控制BUS

RES_VECT    CODE    0x0000
    ;----------設定為輸出接腳------
INITIAL_IO:	
    MOVLB     .17
    MOVLW     0X70
    MOVWF     OSCCON1
    MOVLW	0X04
    MOVWF	OSCFRQ
    MOVLB	.0
    CLRF	IDX	
    MOVLW	0XFC
    MOVWF	TRISC 
    CLRF	TRISD
;---------LCD初始設定---------
INITLCD:              
    CALL      LCD_CLEAR
    CALL      LCD_CURSOR
    CALL      LCD_ENTRY
    CALL      LCD_ONOFF
    CALL      LCD_FUNC	
    GOTO     LCD_DATA1	

;---LCD CLEAR ;CLEAR LCD DDRAM(清除顯示器)---
LCD_CLEAR:      
    MOVLW     B'00000001'
    MOVWF     LCD_DATA
    CALL	  WRIR
    RETURN		

;---LCD CURSOR ;CURSOR GO HOME(游標歸位)---
LCD_CURSOR:  
    MOVLW     B'00000010'
    MOVWF     LCD_DATA
    CALL      WRIR
    RETURN

;---LCD ENTRY MODE(進入模式設定)-----------
LCD_ENTRY: 
    MOVLW     B'00000110'
    MOVWF     LCD_DATA
    CALL	  WRIR
    RETURN

;---LCD (顯示) ON/OFF ------------
LCD_ONOFF:     
    MOVLW	  B'00001100'	 
    MOVWF     LCD_DATA
    CALL	  WRIR
    RETURN

;---LCD FUNCTION SETUP(功能設定)---
LCD_FUNC:    
    MOVLW     B'00111000'
    MOVWF     LCD_DATA
    CALL	  WRIR
    RETURN

;--- SEND LCD DATA ;DISPLAY ON LCD PANEL -------------  

LCD_DATA1:	
    MOVF	  IDX,W		;查表	
    CALL	  TABLE		
    MOVWF	  LCD_DATA
    CALL	  WRDR
    INCF	  IDX,F		;查表指標加1
    MOVLW	  .10		;指標是否指到第五筆
    SUBWF	  IDX,W		
    BTFSS	  STATUS,Z	
    GOTO	  LCD_DATA1	;否,跳到LCD_DATA1執行	
    MOVLW	  B'11000000'	;是,LCD換行顯示		
    MOVWF	  LCD_DATA
    CALL	  WRIR			
LCD_DATA2:	
    MOVF	  IDX,W		;查表
    CALL	  TABLE	
    MOVWF	  LCD_DATA
    CALL	  WRDR		
    INCF	  IDX,F		;查表指標加1
    MOVLW	  .21
    SUBWF	  IDX,W		
    BTFSS	  STATUS,Z	;指標是否指到最後一筆
    GOTO	  LCD_DATA2	;否,跳到LCD_DATA2執行		
WAIT:		NOP			
    GOTO	  WAIT			
;---指令寫入暫存器--------------
WRIR:	
    BCF		LCD_CMD,LCD_RS
    MOVF	LCD_DATA,W
    MOVWF	LCD_BUS
    BSF		LCD_CMD,LCD_EN
    CALL	DELAY
    BCF		LCD_CMD,LCD_EN
    RETURN
;---資料寫入暫存器--------------
WRDR:	
    BSF		LCD_CMD,LCD_RS
    MOVF	LCD_DATA,W
    MOVWF	LCD_BUS
    BSF		LCD_CMD,LCD_EN
    CALL	DELAY
    BCF		LCD_CMD,LCD_EN
    RETURN
;---LCD 延時副程式--------------
DELAY: 	        
    MOVLW	.8
    MOVWF	D2
DLY1:		
    MOVLW	.249
    MOVWF	D1
DLY2:		
    DECFSZ	D1,F
    GOTO	DLY2
    DECFSZ	D2,F
    GOTO	DLY1		
    RETURN

TABLE:		
    ADDWF	PCL,F
    RETLW	0X20	
    RETLW	0X20
    RETLW	0X20
    RETLW	0X20
    RETLW	0X20
    RETLW	0X48
    RETLW	0X45
    RETLW	0X4C
    RETLW	0X4C
    RETLW	0X4F
    RETLW	0X20	
    RETLW	0X20
    RETLW	0X20
    RETLW	0X20
    RETLW	0X57
    RETLW	0X45
    RETLW	0X4C
    RETLW	0X43
    RETLW	0X4F
    RETLW	0X4D
    RETLW	0X45
    END	


