﻿#include <pic16f18877.h>
#include <stdint.h>
#include <stdbool.h>

#pragma config FEXTOSC = HS     // External Oscillator mode selection bits (HS (crystal oscillator) above 4MHz; PFM set to high power)
#pragma config RSTOSC = EXT1X    // Power-up default value for COSC bits->EXTOSC operating per FEXTOSC bits
#pragma config BOREN = ON       // Brown-out reset enable bits (Brown-out Reset Enabled, SBOREN bit is ignored)
#pragma config WDTE = OFF       // WDT operating mode (WDT Disabled, SWDTEN is ignored)
#pragma config CP = OFF         // UserNVM Program memory code protection bit (Program Memory code protection disabled)
#pragma config CPD = OFF        // DataNVM code protection bit (Data EEPROM code protection disabled)

void Initialize(void);
void __interrupt() TMR0_Interrupt (void);

void main(void) {
    Initialize();
    
    while(1){
        
    }
    
    return;
}

void Initialize(void){
    OSCCON1 = 0x70;
    OSCFRQ = 0x04;

    TRISD = 0x00;
    LATD = 0x00;

    //Enable global interrupts.
    INTCONbits.GIE = 1; 
    //Enable peripheral interrupts.
    INTCONbits.PEIE = 1;

    // T0OUTPS 1:1; T0EN enabled; T016BIT 8-bit; 
    T0CON0 = 0x80;
    // T0CS FOSC/4; T0CKPS 1:4096; T0ASYNC synchronised; 
    T0CON1 = 0x4C;
    // TMR0H 219; 
    TMR0H = 0xDB;
    // TMR0L 0; 
    TMR0L = 0x00;
    // Clear Interrupt flag before enabling the interrupt
    PIR0bits.TMR0IF = 0;
    // Enabling TMR0 interrupt.
    PIE0bits.TMR0IE = 1;  
}

void __interrupt() TMR0_Interrupt (void){
    // Clear Interrupt flag before enabling the interrupt
    PIR0bits.TMR0IF = 0;
    
    LATD = ~LATD;
}