﻿#include <pic16f18877.h>
#include <stdint.h>
#include <stdbool.h>

#pragma config FEXTOSC = HS     // External Oscillator mode selection bits (HS (crystal oscillator) above 4MHz; PFM set to high power)
#pragma config RSTOSC = EXT1X    // Power-up default value for COSC bits->EXTOSC operating per FEXTOSC bits
#pragma config BOREN = ON       // Brown-out reset enable bits (Brown-out Reset Enabled, SBOREN bit is ignored)
#pragma config WDTE = OFF       // WDT operating mode (WDT Disabled, SWDTEN is ignored)
#pragma config CP = OFF         // UserNVM Program memory code protection bit (Program Memory code protection disabled)
#pragma config CPD = OFF        // DataNVM code protection bit (Data EEPROM code protection disabled)

void Initialize(void);
void __interrupt() TMR1_Interrupt (void);

void main(void) {
    Initialize();
    
    while(1){
        
    }
    
    return;
}

void Initialize(void){
    OSCCON1 = 0x70;
    OSCFRQ = 0x04;

    TRISD = 0x00;
    LATD = 0x00;
    
    //Enable global interrupts.
    INTCONbits.GIE = 1; 
    //Enable peripheral interrupts.
    INTCONbits.PEIE = 1;

    // CKPS 1:8; nT1SYNC synchronize; TMR1ON enabled; T1RD16 disabled; 
    T1CON = 0x31;
    //CS FOSC/4; 
    T1CLK = 0x01;
    //TMR1H 0; 
    TMR1H = 0x00;
    //TMR1L 0; 
    TMR1L = 0x00;
    // Clearing IF flag before enabling the interrupt.
    PIR4bits.TMR1IF = 0;
    // Enabling TMR1 interrupt.
    PIE4bits.TMR1IE = 1;
}

void __interrupt() TMR1_Interrupt (void){
    // Clear Interrupt flag before enabling the interrupt
    PIR4bits.TMR1IF = 0;
    
    LATD = ~LATD;
}