﻿#include <pic16f18877.h>
#include <stdint.h>
#include <stdbool.h>


#pragma config FEXTOSC = HS     // External Oscillator mode selection bits (HS (crystal oscillator) above 4MHz; PFM set to high power)
#pragma config RSTOSC = EXT1X    // Power-up default value for COSC bits->EXTOSC operating per FEXTOSC bits
#pragma config BOREN = ON       // Brown-out reset enable bits (Brown-out Reset Enabled, SBOREN bit is ignored)
#pragma config WDTE = OFF       // WDT operating mode (WDT Disabled, SWDTEN is ignored)
#pragma config CP = OFF         // UserNVM Program memory code protection bit (Program Memory code protection disabled)
#pragma config CPD = OFF        // DataNVM code protection bit (Data EEPROM code protection disabled)

void Initialize(void);
void __interrupt() TMR1_Interrupt(void);
void Set_SEG7(unsigned char num);
uint8_t count = 0, count2 = 0, n = 0;

void main(void) {
    Initialize();

    while (1) {

    }

    return;
}

void Initialize(void) {
    OSCCON1 = 0x70;
    OSCFRQ = 0x04;

    TRISB = 0xF9;
    TRISD = 0x00;
    LATD = 0x00;

    //Enable global interrupts.
    INTCONbits.GIE = 1;
    //Enable peripheral interrupts.
    INTCONbits.PEIE = 1;

    // T0OUTPS 1:1; T0EN enabled; T016BIT 8-bit; 
    T0CON0 = 0x80;
    // T0CS FOSC/4; T0CKPS 1:128; T0ASYNC synchronised; 
    T0CON1 = 0x47;
    // TMR0H 233; 
    TMR0H = 0xE9;
    // TMR0L 0; 
    TMR0L = 0x00;
    // Clear Interrupt flag before enabling the interrupt
    PIR0bits.TMR0IF = 0;
    // Enabling TMR0 interrupt.
    PIE0bits.TMR0IE = 1;

    // CKPS 1:8; nT1SYNC synchronize; TMR1ON enabled; T1RD16 disabled; 
    T1CON = 0x31;
    //CS FOSC/4; 
    T1CLK = 0x01;
    //TMR1H 109; 
    TMR1H = 0x6D;
    //TMR1L 132; 
    TMR1L = 0x84;
    // Clearing IF flag before enabling the interrupt.
    PIR4bits.TMR1IF = 0;
    // Enabling TMR1 interrupt.
    PIE4bits.TMR1IE = 1;
}

void __interrupt() TMR1_Interrupt(void) {
    // Clear Interrupt flag before enabling the interrupt
    if (PIR0bits.TMR0IF) {
        PIR0bits.TMR0IF = 0;

        if (n == 0) {
            LATBbits.LATB1 = 0;
            LATBbits.LATB2 = 1;
            Set_SEG7(count / 10);
            n = 1;
        } else {
            LATBbits.LATB1 = 1;
            LATBbits.LATB2 = 0;
            Set_SEG7(count % 10);
            n = 0;
        }
        if(count>=100)count=0;
    }
    if (PIR4bits.TMR1IF) {
        PIR4bits.TMR1IF = 0;
        TMR1H = 0x6D;
        TMR1L = 0x84;
        if (++count2 >= 5) {
            count2 = 0;
            count++;
        }
    }
}

void Set_SEG7(uint8_t num) {
    switch (num) {
        case 0:
            LATD = 0b00111111;
            break;
        case 1:
            LATD = 0b00000110;
            break;
        case 2:
            LATD = 0b01011011;
            break;
        case 3:
            LATD = 0b01001111;
            break;
        case 4:
            LATD = 0b01100110;
            break;
        case 5:
            LATD = 0b01101101;
            break;
        case 6:
            LATD = 0b01111101;
            break;
        case 7:
            LATD = 0b00000111;
            break;
        case 8:
            LATD = 0b01111111;
            break;
        case 9:
            LATD = 0b01101111;
            break;
    }
}