﻿#include <pic16f18877.h>
#include <stdint.h>
#include <stdbool.h>

#pragma config FEXTOSC = HS     // External Oscillator mode selection bits (HS (crystal oscillator) above 4MHz; PFM set to high power)
#pragma config RSTOSC = EXT1X    // Power-up default value for COSC bits->EXTOSC operating per FEXTOSC bits
#pragma config BOREN = ON       // Brown-out reset enable bits (Brown-out Reset Enabled, SBOREN bit is ignored)
#pragma config WDTE = OFF       // WDT operating mode (WDT Disabled, SWDTEN is ignored)
#pragma config CP = OFF         // UserNVM Program memory code protection bit (Program Memory code protection disabled)
#pragma config CPD = OFF        // DataNVM code protection bit (Data EEPROM code protection disabled)

void Initialize(void);
uint16_t ADCC_GetSingleConversion(uint16_t channel);
void Delay_x_mS(int N_mS);

uint8_t data;

void main(void) {
    Initialize();
    LATD = 0x01;

    while (1) {
        while (LATDbits.LATD7 == 0) {
            LATD = LATD << 1;
            data = ADCC_GetSingleConversion(0x3) & 0x00FF;
            Delay_x_mS(data);
        }
        while (LATDbits.LATD0 == 0) {
            LATD = LATD >> 1;
            data = ADCC_GetSingleConversion(0x3) & 0x00FF;
            Delay_x_mS(data);
        }
    }

    return;
}

void Initialize(void) {
    OSCCON1 = 0x70;
    OSCFRQ = 0x04;

    TRISD = 0x00;
    LATD = 0x00;

    // ADCCS FOSC/128; 
    ADCLK = 0x3F;
    // ADGO stop; ADFM left; ADON enabled; ADCONT disabled; ADCS FOSC/ADCLK; 
    ADCON0 = 0x80;
}

uint16_t ADCC_GetSingleConversion(uint16_t channel) {
    // select the A/D channel
    ADPCH = channel;
    // Turn on the ADC module
    ADCON0bits.ADON = 1;
    //Disable the continuous mode.
    ADCON0bits.ADCONT = 0;
    // Start the conversion
    ADCON0bits.ADGO = 1;
    // Wait for the conversion to finish
    while (ADCON0bits.ADGO);
    // Conversion finished, return the result
    return ADRESH;
}

void Delay_x_mS(int N_mS) //延遲副程式 (自設變數)
{
    int Loop_mS, Del_1mS; //自設變數

    for (Loop_mS = 0; Loop_mS < N_mS; Loop_mS++)//(條件初始值;條件;運算式) 
    {
        for (Del_1mS = 0; Del_1mS < 199; Del_1mS++);
    }
}