#include <pic16f18877.h>
#include <stdint.h>
#include <stdbool.h>

#pragma config FEXTOSC = HS     // External Oscillator mode selection bits (HS (crystal oscillator) above 4MHz; PFM set to high power)
#pragma config RSTOSC = EXT1X    // Power-up default value for COSC bits->EXTOSC operating per FEXTOSC bits
#pragma config BOREN = ON       // Brown-out reset enable bits (Brown-out Reset Enabled, SBOREN bit is ignored)
#pragma config WDTE = OFF       // WDT operating mode (WDT Disabled, SWDTEN is ignored)
#pragma config CP = OFF         // UserNVM Program memory code protection bit (Program Memory code protection disabled)
#pragma config CPD = OFF        // DataNVM code protection bit (Data EEPROM code protection disabled)

void Initialize(void);
void __interrupt() CCP5_Interrupt(void);

void main(void) {
    Initialize();

    while (1) {
        ADCON0bits.ADGO = 1;
        while (ADCON0bits.ADGO);
        CCPR1H = ADRESH;
    }

    return;
}

void Initialize(void) {
    OSCCON1 = 0x70;
    OSCFRQ = 0x04;

    TRISC = 0xFB;
    TRISD = 0x00;
    LATD = 0x00;
    ANSELA = 0xFD;
    TRISA = 0x02;

    //Enable global interrupts.
    INTCONbits.GIE = 1;
    //Enable peripheral interrupts.
    INTCONbits.PEIE = 1;

    // CKPS 1:8; nT1SYNC synchronize; TMR1ON enabled; T1RD16 disabled; 
    T1CON = 0x01;
    //RA2->TMR1:T1CKI;    
    T1CKIPPS = 0x01;

    // MODE Rising edge; EN enabled; CCP5FMT right_aligned; 
    CCP5CON = 0x81;    
    // CCP5CTS CCP5 pin; 
    CCP5CAP = 0x00; //no
 
    CCPR5L = 0x64;
    CCPR5H = 0x00;
    // Selecting Timer 1
    CCPTMRS1bits.C5TSEL = 0x1;
    // Clear the CCP5 interrupt flag
    PIR6bits.CCP5IF = 0;
    // Enable the CCP5 interrupt
    PIE6bits.CCP5IE = 1;


    // T2CS FOSC/4; 
    T2CLKCON = 0x01;
    // T2PSYNC Not Synchronized; T2MODE Software control; T2CKPOL Rising Edge; T2CKSYNC Not Synchronized; 
    T2HLT = 0x00;//no
    // T2RSEL T2CKIPPS pin; 
    T2RST = 0x00;//no
    // PR2 255; 
    T2PR = 0xFF;
    // TMR2 0; 
    T2TMR = 0x00;//no
    // Clearing IF flag.
    PIR4bits.TMR2IF = 0;//no
    // T2CKPS 1:1; T2OUTPS 1:1; TMR2ON on; 
    T2CON = 0x80;

    // Set the PWM1 to the options selected in the User Interface
    // MODE PWM; EN enabled; CCP1FMT right_aligned; 
    CCP1CON = 0x8F;
    // Selecting Timer 2
    CCPTMRS0bits.C1TSEL = 0x1;
    //RC2->CCP1:CCP1;    
    RC2PPS = 0x09;

    // set the ADCC to the options selected in the User Interface
    // ADCCS FOSC/128; 
    ADCLK = 0x3F;
    // ADGO stop; ADFM left; ADON enabled; ADCONT disabled; ADCS FOSC/ADCLK; 
    ADCON0 = 0x80;
}

void __interrupt() CCP5_Interrupt(void) {
    // Clear the CCP5 interrupt flag
    PIR6bits.CCP5IF = 0;
    
    LATD = ~LATD;
}