#include <pic16f18877.h>
#include <stdint.h>
#include <stdbool.h>

#pragma config FEXTOSC = HS     // External Oscillator mode selection bits (HS (crystal oscillator) above 4MHz; PFM set to high power)
#pragma config RSTOSC = EXT1X    // Power-up default value for COSC bits->EXTOSC operating per FEXTOSC bits
#pragma config BOREN = ON       // Brown-out reset enable bits (Brown-out Reset Enabled, SBOREN bit is ignored)
#pragma config WDTE = OFF       // WDT operating mode (WDT Disabled, SWDTEN is ignored)
#pragma config CP = OFF         // UserNVM Program memory code protection bit (Program Memory code protection disabled)
#pragma config CPD = OFF        // DataNVM code protection bit (Data EEPROM code protection disabled)

void Initialize(void);
void __interrupt() CCP5_Interrupt(void);

void main(void)
{
    Initialize();

    while (1)
    {
        ADCON0bits.ADGO = 1;
        while (ADCON0bits.ADGO);
        CCPR1H = ADRESH;
    }

    return;
}

void Initialize(void)
{
    OSCCON1 = 0x70;
    OSCFRQ = 0x04;
    TRISB = 0x00;
    TRISC = 0xFB;
    TRISD = 0x00;
    LATB = 0x10;
    LATD = 0xFF;
    ANSELA = 0xFD;
    TRISA = 0xFF;

    //Enable global interrupts.
    INTCONbits.GIE = 1;
    //Enable peripheral interrupts.
    INTCONbits.PEIE = 1;

    //CS FOSC/4; 
    T1CLK = 0x01;
    // CKPS 1:1; nT1SYNC synchronize; TMR1ON enabled; T1RD16 disabled; 
    T1CON = 0x31;

    // MODE Rising edge; EN enabled; CCP5FMT right_aligned; 
    CCP5CON = 0x85;
    // Write the compare value
    CCPR5L = 0x00;
    CCPR5H = 0x00;
    // Selecting Timer 1
    CCPTMRS1bits.C5TSEL = 0x1;
    // Clear the CCP5 interrupt flag
    PIR6bits.CCP5IF = 0;
    // Enable the CCP5 interrupt
    PIE6bits.CCP5IE = 1;
    CCP5PPS = 0x01; //RA1->CCP5:CCP5;   


    // T2CS FOSC/4; 
    T2CLKCON = 0x01;
    // T2PSYNC Not Synchronized; T2MODE Software control; T2CKPOL Rising Edge; T2CKSYNC Not Synchronized; 
    T2HLT = 0x00;
    // T2RSEL T2CKIPPS pin; 
    T2RST = 0x00;
    // PR2 255; 
    T2PR = 0xFF;
    // TMR2 0; 
    T2TMR = 0x00;
    // Clearing IF flag.
    PIR4bits.TMR2IF = 0;
    // T2CKPS 1:1; T2OUTPS 1:1; TMR2ON on; 
    T2CON = 0x80;

    // Set the PWM1 to the options selected in the User Interface
    // MODE PWM; EN enabled; CCP1FMT right_aligned; 
    CCP1CON = 0x8F;
    // Selecting Timer 2
    CCPTMRS0bits.C1TSEL = 0x1;
    //RC2->CCP1:CCP1;    
    RC2PPS = 0x09;

    // set the ADCC to the options selected in the User Interface
    // ADCCS FOSC/128; 
    ADCLK = 0x3F;
    // ADGO stop; ADFM left; ADON enabled; ADCONT disabled; ADCS FOSC/ADCLK; 
    ADCON0 = 0x80;
}

void __interrupt() CCP5_Interrupt(void)
{
    // Clear the CCP5 interrupt flag
    PIR6bits.CCP5IF = 0;
      unsigned int value = (CCPR5H << 8) + CCPR5L;
    
    if (value >= 1500)
    {
        LATD = 0b00000110;
    }
    else if (value >= 1400)
    {
        LATD = 0b01011011;
    }
    else if (value >= 1300)
    {
        LATD = 0b01001111;
    }
    else if (value >= 1000)
    {
        LATD = 0b01100110;
    }
    else if (value >= 850)
    {
        LATD = 0b01101101;
    }
    else if (value >= 800)
    {
        LATD = 0b01111101;
    }
    else if (value >= 760)
    {
        LATD = 0b00000111;
    }
    else if (value >= 720)
    {
        LATD = 0b01111111;
    }
    else if (value >= 700)
    {
        LATD = 0b01101111;
    }
    TMR1H = 0;
    TMR1L = 0;
}