﻿#include <pic16f18877.h>
#include <stdint.h>
#include <stdbool.h>

#pragma config FEXTOSC = HS     // External Oscillator mode selection bits (HS (crystal oscillator) above 4MHz; PFM set to high power)
#pragma config RSTOSC = EXT1X    // Power-up default value for COSC bits->EXTOSC operating per FEXTOSC bits
#pragma config BOREN = ON       // Brown-out reset enable bits (Brown-out Reset Enabled, SBOREN bit is ignored)
#pragma config WDTE = OFF       // WDT operating mode (WDT Disabled, SWDTEN is ignored)
#pragma config CP = OFF         // UserNVM Program memory code protection bit (Program Memory code protection disabled)
#pragma config CPD = OFF        // DataNVM code protection bit (Data EEPROM code protection disabled)
void Initialize(void);

void main(void) {
    Initialize();

    while (1) {
        if (!PORTEbits.RE0) {
            CCPR1L = 0x38;
        } else {
            CCPR1L = 0x43;
        }
    }

    return;
}

void Initialize(void) {
    OSCCON1 = 0x70;
    OSCFRQ = 0x04;

    TRISE = 0x07;
    ANSELE = 0x06;
    TRISC = 0xFB;
    RC2PPS = 0x09; //RC2->CCP1:CCP1;    

    // T2CS FOSC/4; 
    T2CLKCON = 0x01;
    // T2PSYNC Not Synchronized; T2MODE Software control; T2CKPOL Rising Edge; T2CKSYNC Not Synchronized; 
    T2HLT = 0x00;
    // T2RSEL T2CKIPPS pin; 
    T2RST = 0x00;
    // PR2 18; 
    T2PR = 0x12;
    // TMR2 0; 
    T2TMR = 0x00;
    // Clearing IF flag.
    PIR4bits.TMR2IF = 0;
    // T2CKPS 1:1; T2OUTPS 1:16; TMR2ON on; 
    T2CON = 0x8F;

    // MODE PWM; EN enabled; CCP1FMT right_aligned; 
    CCP1CON = 0x8F;
    // RH 0; 
    CCPR1H = 0x00;
    // RL 67; 
    CCPR1L = 0x43;
    // Selecting Timer 2
    CCPTMRS0bits.C1TSEL = 0x1;
}