﻿#include <pic16f18877.h>
#include <stdint.h>
#include <stdbool.h>

#pragma config FEXTOSC = HS     // External Oscillator mode selection bits (HS (crystal oscillator) above 4MHz; PFM set to high power)
#pragma config RSTOSC = EXT1X    // Power-up default value for COSC bits->EXTOSC operating per FEXTOSC bits
#pragma config BOREN = ON       // Brown-out reset enable bits (Brown-out Reset Enabled, SBOREN bit is ignored)
#pragma config WDTE = OFF       // WDT operating mode (WDT Disabled, SWDTEN is ignored)
#pragma config CP = OFF         // UserNVM Program memory code protection bit (Program Memory code protection disabled)
#pragma config CPD = OFF        // DataNVM code protection bit (Data EEPROM code protection disabled)

void Initialize(void);

#define LCD_RS LATCbits.LATC0
#define LCD_EN LATCbits.LATC1
#define LCD LATD

void LCD_Init(void);
void LCD_Command(unsigned char cmd);
void LCD_Char(unsigned char dat);
void LCD_String(const char *msg);
void Delay_x_mS( int N_mS );

void main(void) {
    Initialize();
    LCD_Init();

    LCD_String("     HELLO");
    LCD_Command(0xC0);
    LCD_String("    WELCOME");

    while (1) {

    }

    return;
}

void Initialize(void) {
    OSCCON1 = 0x70;
    OSCFRQ = 0x04;

    TRISC = 0xFC;
    TRISD = 0x00;
}

void LCD_Init(void) {
    Delay_x_mS(100);
    LCD_Command(0b00111000);
    LCD_Command(0b00000010);
    LCD_Command(0b00000001); /*clear display screen*/
    LCD_Command(0b00001100); /*display on cursor off*/
    LCD_Command(0b00000110); /*increment cursor (shift cursor to right)*/
}

void LCD_Command(unsigned char cmd) {
    LCD = cmd;
    LCD_RS = 0;
    LCD_EN = 1;
    Delay_x_mS(1);
    LCD_EN = 0;
    Delay_x_mS(3);
}

void LCD_Char(unsigned char dat) {
    LCD = dat;
    LCD_RS = 1;
    LCD_EN = 1;
    Delay_x_mS(1);
    LCD_EN = 0;
    Delay_x_mS(3);
}

void LCD_String(const char *msg) {
    while ((*msg) != '\0') {
        LCD_Char(*msg);
        msg++;
    }
}

void Delay_x_mS( int N_mS ) //延遲副程式 (自設變數)
{
		int	Loop_mS,Del_1mS;//自設變數
		
		for ( Loop_mS = 0 ; Loop_mS < N_mS ; Loop_mS++ )//(條件初始值;條件;運算式) 
		{
			for (Del_1mS = 0 ; Del_1mS < 199 ; Del_1mS ++ );
					
		}
		 
}