/**
  Generated Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This is the main file generated using PIC10 / PIC12 / PIC16 / PIC18 MCUs

  Description:
    This header file provides implementations for driver APIs for all modules selected in the GUI.
    Generation Information :
        Product Revision  :  PIC10 / PIC12 / PIC16 / PIC18 MCUs - 1.77
        Device            :  PIC16F18877
        Driver Version    :  2.00
 */

/*
    (c) 2018 Microchip Technology Inc. and its subsidiaries. 
    
    Subject to your compliance with these terms, you may use Microchip software and any 
    derivatives exclusively with Microchip products. It is your responsibility to comply with third party 
    license terms applicable to your use of third party software (including open source software) that 
    may accompany Microchip software.
    
    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY 
    IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS 
    FOR A PARTICULAR PURPOSE.
    
    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP 
    HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO 
    THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL 
    CLAIMS IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT 
    OF FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS 
    SOFTWARE.
 */

#include "mcc_generated_files/mcc.h"

/*
                         Main application
 */
#define LCD_RS LATCbits.LATC0
#define LCD_EN LATCbits.LATC1
#define LCD LATD

void LCD_Init(void);
void LCD_Command(unsigned char cmd);
void LCD_Char(unsigned char dat);
void LCD_String(const char *msg);
void Delay_x_mS(int N_mS);

uint8_t Keyboard(void);

uint8_t buffer_send, buffer_read;

#define slaveDeviceAddress  0x8
#define SLAVE_I2C_GENERIC_RETRY_MAX 100
uint16_t dataAddress;
uint8_t *pData;
uint16_t nCount;
uint8_t writeBuffer[3];
uint8_t readBuffer[3];
uint16_t counter, timeOut;

void main(void) {
    // initialize the device
    SYSTEM_Initialize();

    // When using interrupts, you need to set the Global and Peripheral Interrupt Enable bits
    // Use the following macros to:

    // Enable the Global Interrupts
    INTERRUPT_GlobalInterruptEnable();

    // Enable the Peripheral Interrupts
    INTERRUPT_PeripheralInterruptEnable();

    // Disable the Global Interrupts
    //INTERRUPT_GlobalInterruptDisable();

    // Disable the Peripheral Interrupts
    //INTERRUPT_PeripheralInterruptDisable();

    LCD_Init();
    LCD_String("Read char= ");

    I2C1_MESSAGE_STATUS status = I2C1_MESSAGE_PENDING;
    dataAddress = 0x10; // starting EEPROM address 
    nCount = 16; // number of bytes to write

    while (1) {
        // Add your application code
        buffer_send = Keyboard();
        if (buffer_send != '?') {

            //I2C Write Start
            status = I2C1_MESSAGE_PENDING;
            // build the write buffer first
            writeBuffer[0] = buffer_send;

            timeOut = 0;
            while (status != I2C1_MESSAGE_FAIL) {
                // write one byte to SLAVE
                I2C1_MasterWrite(writeBuffer,
                        1,
                        slaveDeviceAddress,
                        &status);

                // wait for the message to be sent or status has changed.
                while (status == I2C1_MESSAGE_PENDING);

                if (status == I2C1_MESSAGE_COMPLETE)
                    break;

                // if status is  I2C1_MESSAGE_ADDRESS_NO_ACK,
                //               or I2C1_DATA_NO_ACK,
                // The device may be busy and needs more time for the last
                // write so we can retry writing the data, this is why we
                // use a while loop here

                // check for max retry and skip this byte
                if (timeOut == SLAVE_I2C_GENERIC_RETRY_MAX)
                    break;
                else
                    timeOut++;
            }
            //I2C Write End
        }

        //I2C Read Start
        status = I2C1_MESSAGE_PENDING;
        timeOut = 0;
        while (status != I2C1_MESSAGE_FAIL) {
            // read one byte to EEPROM
            I2C1_MasterRead(readBuffer,
                    1,
                    slaveDeviceAddress,
                    &status);

            // wait for the message to be sent or status has changed.
            while (status == I2C1_MESSAGE_PENDING);

            if (status == I2C1_MESSAGE_COMPLETE)
                break;

            // if status is  I2C1_MESSAGE_ADDRESS_NO_ACK,
            //               or I2C1_DATA_NO_ACK,
            // The device may be busy and needs more time for the last
            // write so we can retry writing the data, this is why we
            // use a while loop here

            // check for max retry and skip this byte
            if (timeOut == SLAVE_I2C_GENERIC_RETRY_MAX)
                break;
            else
                timeOut++;
        }
        //if the last transaction failed
        if (status != I2C1_MESSAGE_FAIL) {
            buffer_read = readBuffer[0];
            if (buffer_read != '?') {
                LCD_Command(0x8B);
                LCD_Char(buffer_read);
            }
        }
        //I2C Read End
    }
}

void LCD_Init(void) {
    TRISC = 0xF0;
    TRISD = 0x00;
    TRISE = 0x00;
    ANSELC = 0x0F;
    Delay_x_mS(100);
    LCD_Command(0b00111000);
    LCD_Command(0b00000010);
    LCD_Command(0b00000001); /*clear display screen*/
    LCD_Command(0b00001100); /*display on cursor off*/
    LCD_Command(0b00000110); /*increment cursor (shift cursor to right)*/
}

void LCD_Command(unsigned char cmd) {
    LCD = cmd;
    LCD_RS = 0;
    LCD_EN = 1;
    Delay_x_mS(1);
    LCD_EN = 0;
    Delay_x_mS(3);
}

void LCD_Char(unsigned char dat) {
    LCD = dat;
    LCD_RS = 1;
    LCD_EN = 1;
    Delay_x_mS(1);
    LCD_EN = 0;
    Delay_x_mS(3);
}

void LCD_String(const char *msg) {
    while ((*msg) != '\0') {
        LCD_Char(*msg);
        msg++;
    }
}

void Delay_x_mS(int N_mS) //延遲副程式 (自設變數)
{
    int Loop_mS, Del_1mS; //自設變數

    for (Loop_mS = 0; Loop_mS < N_mS; Loop_mS++)//(條件初始值;條件;運算式) 
    {
        for (Del_1mS = 0; Del_1mS < 199; Del_1mS++);

    }

}

uint8_t Keyboard(void) {
    uint8_t KEY_value = '?';
    LATEbits.LATE0 = 0;
    LATEbits.LATE1 = 1;
    LATEbits.LATE2 = 1;
    Delay_x_mS(1);
    if (!PORTCbits.RC4) {
        Delay_x_mS(1);
        if (!PORTCbits.RC4) {
            KEY_value = '1';
            while (!PORTCbits.RC4);
        }
    } else if (!PORTCbits.RC5) {
        Delay_x_mS(1);
        if (!PORTCbits.RC5) {
            KEY_value = '4';
            while (!PORTCbits.RC5);
        }
    } else if (!PORTCbits.RC6) {
        Delay_x_mS(1);
        if (!PORTCbits.RC6) {
            KEY_value = '7';
            while (!PORTCbits.RC6);
        }
    } else if (!PORTCbits.RC7) {
        Delay_x_mS(1);
        if (!PORTCbits.RC7) {
            KEY_value = '*';
            while (!PORTCbits.RC7);
        }
    }

    LATEbits.LATE0 = 1;
    LATEbits.LATE1 = 0;
    LATEbits.LATE2 = 1;
    Delay_x_mS(1);
    if (!PORTCbits.RC4) {
        Delay_x_mS(1);
        if (!PORTCbits.RC4) {
            KEY_value = '2';
            while (!PORTCbits.RC4);
        }
    } else if (!PORTCbits.RC5) {
        Delay_x_mS(1);
        if (!PORTCbits.RC5) {
            KEY_value = '5';
            while (!PORTCbits.RC5);
        }
    } else if (!PORTCbits.RC6) {
        Delay_x_mS(1);
        if (!PORTCbits.RC6) {
            KEY_value = '8';
            while (!PORTCbits.RC6);
        }
    } else if (!PORTCbits.RC7) {
        Delay_x_mS(1);
        if (!PORTCbits.RC7) {
            KEY_value = '0';
            while (!PORTCbits.RC7);
        }
    }

    LATEbits.LATE0 = 1;
    LATEbits.LATE1 = 1;
    LATEbits.LATE2 = 0;
    Delay_x_mS(1);
    if (!PORTCbits.RC4) {
        Delay_x_mS(1);
        if (!PORTCbits.RC4) {
            KEY_value = '3';
            while (!PORTCbits.RC4);
        }
    } else if (!PORTCbits.RC5) {
        Delay_x_mS(1);
        if (!PORTCbits.RC5) {
            KEY_value = '6';
            while (!PORTCbits.RC5);
        }
    } else if (!PORTCbits.RC6) {
        Delay_x_mS(1);
        if (!PORTCbits.RC6) {
            KEY_value = '9';
            while (!PORTCbits.RC6);
        }
    } else if (!PORTCbits.RC7) {
        Delay_x_mS(1);
        if (!PORTCbits.RC7) {
            KEY_value = '#';
            while (!PORTCbits.RC7);
        }
    }

    return KEY_value;
}
/**
 End of File
 */